/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.collections.stream.Streams;
import net.sf.staccatocommons.defs.Evaluable;
import net.sf.staccatocommons.lang.predicate.AbstractPredicate;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

/**
 *
 * CompositeTypeConverter
 *
 * @author flbulgarelli
 */
public class CompositeTypeConverter implements TypeConverter {

    private final Iterable<TypeConverter> typeConverters;

    public CompositeTypeConverter(@NonNull final Iterable<TypeConverter> typeConverters) {
        this.typeConverters = typeConverters;
    }

    @ForceRestrictions
    public boolean canConvert(@NonNull final Class<?> sourceType, @NonNull final Class<?> destinationType) {
        return Streams.from(typeConverters) //
                .any(converterCanConvert(sourceType, destinationType));
    }

    @ForceRestrictions
    public <A> A convert(@NonNull final Object value, @NonNull final Class<A> destinationType) {
        TypeConverter converter = findConverter(value, destinationType);
        Ensure.that(converter != null, "No converter found that can convert %s to %s", value, destinationType);
        return converter.convert(value, destinationType);
    }

    protected <A> TypeConverter findConverter(final Object value, final Class<A> destinationType) {
        return Streams.from(typeConverters) //
                .findOrNull(converterCanConvert(value.getClass(), destinationType));
    }

    private Evaluable<? super TypeConverter> converterCanConvert(final Class<?> sourceType, final Class<?> destinationType) {
        return new AbstractPredicate<TypeConverter>() {
            @Override
            public boolean eval(final TypeConverter arg0) {
                return arg0.canConvert(sourceType, destinationType);
            }
        };
    }

}
