/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.restrictions.Constant;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

/**
 *
 * IdentityTypeConverter
 *
 * @author flbulgarelli
 */
public class IdentityTypeConverter implements TypeConverter {

    @ForceRestrictions
    public boolean canConvert(@NonNull final Class<?> sourceType, @NonNull final Class<?> destinationType) {
        return destinationType.isAssignableFrom(sourceType);
    }

    @SuppressWarnings("unchecked")
    public <A> A convert(final Object value, final Class<A> destinationType) {
        Ensure.that(value == null || canConvert(value.getClass(), destinationType), //
                "%s is not of type %s", value, destinationType);
        return (A) value;
    }

    @Constant
    public static TypeConverter converter() {
        return new IdentityTypeConverter();
    }

}
