/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter.impl;

import ar.com.zauber.commons.mom.converter.TypeConverter;

import java.util.Date;
import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import net.sf.staccatocommons.restrictions.Constant;

/**
 * {@link TypeConverter} that can convert {@link Date}s into {@link XMLGregorianCalendar}s
 * based on its time
 *
 * @author flbulgarelli
 */
public class DateToXMLGregorianCalendarTypeConverter implements TypeConverter {

    private final DatatypeFactory datatypeFactory;
    {
        try {
            datatypeFactory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new AssertionError(e);
        }
    }

    protected XMLGregorianCalendar toGregorianCalendar(final Date openingBalanceDate) {
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(openingBalanceDate);
        return datatypeFactory.newXMLGregorianCalendar(cal);
    }

    public boolean canConvert(final Class<?> sourceType, final Class<?> destinationType) {
        return destinationType == XMLGregorianCalendar.class && Date.class.isAssignableFrom(sourceType);
    }

    @SuppressWarnings("unchecked")
    public <A> A convert(final Object value, final Class<A> destinationType) {
        return (A) toGregorianCalendar((Date) value);
    }

    /**
     * Answers a constant instance of {@link DateToXMLGregorianCalendarTypeConverter}
     * @return a constant {@link DateToXMLGregorianCalendarTypeConverter}
     */
    @Constant
    public static TypeConverter converter() {
        return new DateToXMLGregorianCalendarTypeConverter();
    }
}
