/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter.impl;

import ar.com.zauber.commons.mom.converter.TypeConverter;

import net.sf.staccatocommons.restrictions.Constant;

/**
 * {@link TypeConverter} that can convert String into enums based on its enum
 * value
 *
 * @author flbulgarelli
 */
public class StringToEnumTypeConverter implements TypeConverter {

    public boolean canConvert(Class<?> sourceType, Class<?> destinationType) {
        return sourceType == String.class && destinationType.isEnum();
    }

    @SuppressWarnings("unchecked")
    public <A> A convert(final Object value, final Class<A> destinationType) {
        return (A) Enum.valueOf((Class) destinationType, (String) value);
    }

    /**
     * Answers a constant instance of {@link StringToEnumTypeConverter}
     * @return a constant {@link StringToEnumTypeConverter}
     */
    @Constant
    public static StringToEnumTypeConverter converter() {
        return new StringToEnumTypeConverter();
    }

}
