/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.internal;

import ar.com.zauber.commons.mom.MapObjectMapper;

import java.util.Iterator;

import net.sf.staccatocommons.collections.stream.Streams;
import net.sf.staccatocommons.restrictions.check.NonNull;

import org.apache.commons.beanutils.BeanMap;

/**
 * A delayed map that projects an object into a map. The logic is the following:
 * Null attributes are converted into null, mappeable arrays object attributes
 * are converted into collections of maps, mappeable object attributes are
 * converted into maps, and any other attribute is left unchanged.
 */
@SuppressWarnings("unchecked")
public class AttributesMap extends BeanMap {
    private final MapObjectMapper mom;

    /**
     * Creates the {@link AttributesMap}
     *
     * @param object
     * @param mom
     */
    public AttributesMap(final Object object, @NonNull final MapObjectMapper mom) {
        super(object);
        this.mom = mom;
    }

    @Override
    public final Object get(final Object name) {
        return name.equals("class") ? null : transform(super.get(name));
    }

    @Override
    public boolean containsKey(final Object name)
    {
        return !"class".equals(name) && super.containsKey(name);
    }


    @Override
    public Iterator keyIterator()
    {
        return Streams.from(super.keyIterator()).skip("class").iterator();
    }

    protected Object transform(final Object value) {
        return mom.map(value);
    }

    @Override
    public final String toString() {
        /*
         * This is rough copy paste of the original toString implementation of
         * AbstractMap, that BeanMap overrides
         */
        Iterator<java.util.Map.Entry<String, Object>> i = entrySet().iterator();
        if (!i.hasNext()) {
            return "{}";
        }
        StringBuilder sb = new StringBuilder();
        sb.append('{');
        for (;;) {
            java.util.Map.Entry<String, Object> e = i.next();
            String key = e.getKey();
            Object value = e.getValue();
            sb.append(key);
            sb.append('=');
            sb.append(value == this ? "(this Map)" : value);
            if (!i.hasNext()) {
                return sb.append('}').toString();
            }
            sb.append(", ");
        }
    }

}
